#!/usr/bin/env python
import sys
from fractions import Fraction
import math

def mirror(p, a1, a2):
    dx = a2.x - a1.x
    dy = a2.y - a1.y

    a = (dx * dx - dy * dy) / (dx * dx + dy*dy)
    b = 2 * dx * dy / (dx*dx + dy*dy)

    x2 = a * (p.x - a1.x) + b*(p.y - a1.y) + a1.x
    y2 = b * (p.x - a1.x) - a*(p.y - a1.y) + a1.y

    return Point(x2, y2, p.src)

def positive(poly):
    #print poly
    s=0
    for i in range(len(poly)-1):
        s+=(poly[i+1].x - poly[i].x)*(poly[i+1].y + poly[i].y);
    return s>0

def trace2poly(trace):
    # FIXME: check this, bugs?
    poly=list(trace[0].ps[:2])
    for i in trace[1:]:
        for j in i.ps[:2]:
            if j not in poly:
                poly.append(j)
    poly.append(trace[0].ps[0])
    # print 'poly', poly
    return poly

def intersect(e1,e2):
    s1= e1.ps[0].x - e1.ps[1].x, e1.ps[0].y - e1.ps[1].y
    s2= e2.ps[1].x - e2.ps[0].x, e2.ps[1].y - e2.ps[0].y

    s = (-s1[1] * (e1.ps[1].x - e2.ps[0].x) + s1[0] * (e1.ps[1].y - e2.ps[0].y)) / (-s2[0] * s1[1] + s1[0] * s2[1])
    t = ( s2[0] * (e1.ps[1].y - e2.ps[0].y) - s2[1] * (e1.ps[1].x - e2.ps[0].x)) / (-s2[0] * s1[1] + s1[0] * s2[1])

    if (s >= 0 and s <= 1 and t >= 0 and t <= 1):
        return Point(e1.ps[1].x + (t * s1[0]), e1.ps[1].y + (t * s1[1]))
        
    return None

def splitedge(edge,points):
    
    sp=list(edge.ps[:2])
    
    points=[edge.ps[0]]+points+[edge.ps[1]]
    points.sort()
    ret=[]
    for i in xrange(len(points)-1):
        ret.append(Edge(points[i],points[i+1]))
    return ret

class Frac(Fraction):
    def __repr__(self):
        return '%d/%d' % (self.numerator, self.denominator)

def format_frac(x):
    if x.denominator == 1:
        return str(x.numerator)
    return '%d/%d' % (x.numerator, x.denominator)

class Point(object):
    def __init__(self, x, y, src=-1):
        self.x = x
        self.y = y
        self.src = src
    def __getitem__(self,i):
        if i==0:
            return self.x
        elif i==1:
            return self.y
        else:
            return 1/0
    def __eq__(self, p):
        return (self.x == p.x) and (self.y == p.y) 

    def __cmp__(self, p):
        return cmp((self.x, self.y), (p.x, p.y))

    def __hash__(self):
        return hash((self.x, self.y))

    def __repr__(self):
        return '<%s %s %s>' % (format_frac(self.x), format_frac(self.y), self.src)

    def format(self):
        return '%s,%s' % (format_frac(self.x), format_frac(self.y))

    def format_flt(self):
        return '%f %f' % (float(self.x), float(self.y))
        
        

class Edge(object):
    def __init__(self, ps0, ps1, zindex=0):
        self.ps = (ps0, ps1, zindex)
        assert ps0 != ps1

    def is_parallel(self, edge):
        p1x = self.ps[0].x - self.ps[1].x
        p1y = self.ps[0].y - self.ps[1].y
        e1x = edge.ps[0].x - edge.ps[1].x
        e1y = edge.ps[0].y - edge.ps[1].y

        if p1x == 0:
            return e1x == 0

        if p1y == 0:
            return e1y == 0

        if (e1y == 0) or (e1x == 0):
            return False
        k1 = p1x/p1y
        k2 = e1x/e1y
        return k1 == k2

    def canonical(self):
        if self.ps[0] < self.ps[1]:
            return self
        else:
            return Edge(self.ps[1], self.ps[0], self.ps[2])

    def mirror_point(self, p):
        return mirror(p, self.ps[0], self.ps[1])

    def __hash__(self):
        return hash(self.ps)

    def __eq__(self, other):
        return self.ps == other.ps

    def __cmp__(self, other):
        return cmp(self.ps, other.ps)

    def eq_undirectional(self, other):
        return (self.ps == other.ps) or \
               (self.ps[1], self.ps[0], self.ps[2]) == other.ps
    def eq_geo(self, other):
        return (self.ps[:2]== other.ps[:2]) or \
               (self.ps[1], self.ps[0]) == other.ps[:2]
    

    def __repr__(self):
        return 'Edge(%s, %s, zi=%s)' % (self.ps[0], self.ps[1], self.ps[2])
    def scalar(self,edge):
        scal=0
        for i in range(2):
            scal+=(self.ps[1][i]-self.ps[0][i])*(edge.ps[1][i]-edge.ps[0][i])
        return scal
    def length(self):
        return math.sqrt(self.scalar(self))
class Polygon(object):
    def __init__(self, edges, area=None):
        self.edges = edges
        self.assert_chaining()
        self.area = self.calc_area() if area is None else area

    @classmethod
    def from_points(self, points):
        edges = []
        for i in range(len(points)):
            edges.append(Edge(points[i], points[(i+1) % len(points)]))
        return Polygon(edges)

    def __repr__(self):
        return 'Poly(n=%s, %s, area=%s)' % (len(self.edges), self.edges, self.area)

    def assert_chaining(self):
        for i, e0 in enumerate(self.edges):
            e1 = self.edges[(i + 1) % len(self.edges)]
            assert e0.ps[1] == e1.ps[0]

    def bounding_rect(self):
        xmin = self.edges[0][0]
        xmax = xmin
        ymin = self.edges[1][0]
        ymax = xmin
        for e in self.edges[1:]:
            xmin = min(xmin, e[0].x)
            xmax = max(xmax, e[0].x)
            ymin = min(xmin, e[0].y)
            ymax = max(xmax, e[0].y)
        return (xmin, ymin), (xmax, ymax)

    def calc_area(self):
        # magic!
        s = sum(e.ps[0].x * e.ps[1].y - e.ps[1].x * e.ps[0].y
                for e in self.edges)
        return abs(s) / 2

    def neigbors(self):
        ret = []
        for i in self.edges:
            ret.extend(filter(lambda x: x != self, i.polys))
        return ret

    def is_positive(self):
        s = 0
        for i in range(len(self.edges) - 1):
            s += (self.edges[i+1].x - self.edges[i].x) * \
                 (self.edges[i+1].y + self.edges[i].y)
        return s > 0

def atan3(edge,i):
    if edge.ps[0]==i:
        return math.atan2(edge.ps[1].y-edge.ps[0].y,edge.ps[1].x-edge.ps[0].x)
    else:
        return math.atan2(edge.ps[0].y-edge.ps[1].y,edge.ps[0].x-edge.ps[1].x)

class State():
    def __init__(self, source):
        if isinstance(source, Task):
            self.polys = []
            self.edges = map(lambda x:Edge(x[0],x[1]),source.skeleton)
            newedges = []
            for i in self.edges:
                splitpoints=[]
                for j in self.edges:
                    if i==j:
                        continue
                    if (j.ps[0] in i.ps[:2]) or (j.ps[1] in i.ps[:2]):
                        continue
                    if i.is_parallel(j):
                        continue
                    k=intersect(i,j)
                    if k:
                        
                        if k not in i.ps[:2]:
                            if not k in splitpoints:
                                splitpoints.append(k)
                if splitpoints:
                    
                    newedges.extend(splitedge(i,splitpoints))
                    
                else:
                    newedges.append(i)
            points=set()
            for i in newedges:
                points.add(i.ps[0])
                points.add(i.ps[1])
            pedges={}
            for i in points:
                pedges[i]=[]
                for j in newedges:
                    if i in j.ps[:2]:
                        pedges[i].append(j)
            for i in pedges.keys():
                pedges[i].sort(key=lambda x:atan3(x,i))
            self.edges=newedges
            self.pedges=pedges
            for i in self.edges:
                currpoint=i.ps[1]
                curredge=i
                trace=[curredge]
                while True:
                    curredges=pedges[currpoint]
                    if curredges[-1]==curredge:
                        nextedge=curredges[0]
                    else:
                        nextedge=curredges[curredges.index(curredge)+1]
                    if nextedge==i:
                        dupe=False
                        for j in self.polys:
                            if set(i.canonical() for i in j.edges) == \
                               set(i.canonical() for i in trace):
                                dupe=True
                        if not dupe and positive(trace2poly(trace)):
                            if not set(trace2poly(trace)) in map(set,filter(positive,source.polys)):
                                # print trace
                                polygon = Polygon.from_points(trace2poly(trace)[:-1])
                                # print 'append', polygon
                                self.polys.append(polygon)
                            
                        break
                    trace.append(nextedge)
                    if nextedge.ps[0]==currpoint:
                        currpoint=nextedge.ps[1]
                    else:
                        currpoint=nextedge.ps[0]
                    curredge=nextedge
            # print len(self.polys)

class Task(object):
    def __init__(self, inf=sys.stdin):
        self.polycount = int(inf.readline())
        self.polys = []
        for _ in xrange(self.polycount):
            cpoly = []
            vcount = int(inf.readline())
            for _ in xrange(vcount):
                cpoly.append(Point(*map(Frac, inf.readline().split(','))))
            self.polys.append(cpoly)

        self.skeleton = []
        self.skelecount = int(inf.readline())
        for _ in range(self.skelecount):
            self.skeleton.append(
                tuple(map(lambda x: Point(*map(Frac,x.split(","))),inf.readline().split(" ")
                ))
            )
         
    def __str__(self):
        return "\n".join(map(lambda x:`x`,self.polys))+"\n\n"+"\n".join(map(lambda x:`x`,self.skeleton))


def test_area():
    p1 = Point(Frac("2.0"), Frac("1.0"))
    p2 = Point(Frac("2.0"), Frac("3.0"))
    p3 = Point(Frac("4.0"), Frac("3.0"))
    p4 = Point(Frac("3.0"), Frac("2.0"))
    p5 = Point(Frac("4.0"), Frac("1.0"))
    poly = Polygon.from_points([p1, p2, p3, p4, p5])
    assert poly.calc_area() == 3
    
def test():
    test_area()
    p1 = Point(Frac("2.0"), Frac("1.0"))
    p2 = Point(Frac("4.0"), Frac("1.0"))
    p3 = Point(Frac("5.0"), Frac("1.0"))
    p4 = Point(Frac("5.0"), Frac("2.0"))
    p5 = Point(Frac("5.2"), Frac("2.1"))
    p6 = Point(Frac("2.0"), Frac("8.0"))
    p7 = Point(Frac("2.0"), Frac("2.0"))
    p8 = Point(Frac("2.0"), Frac("3.0"))
    p9 = Point(Frac("3.0"), Frac("2.0"))
    p10 = Point(Frac("3.0"), Frac("3.0"))
    p11 = Point(Frac("2.5"), Frac("2.5"))
    assert Edge(p1, p2).is_parallel(Edge(p2, p3)) == True
    assert Edge(p1, p2).is_parallel(Edge(p3, p4)) == False
    assert Edge(p1, p6).is_parallel(Edge(p3, p4)) == True
    assert Edge(p1, p2) == Edge(p1, p2)
    assert Edge(p1, p3) != Edge(p1, p2)
    assert Edge(p1, p2) == Edge(p1, p2)
    assert intersect(Edge(p7,p10),Edge(p8,p9)) == p11
    assert cmp(Edge(p1, p2), Edge(p1, p2)) == 0
    
if __name__ == '__main__':
    test()
