#!/usr/bin/python

import Queue
import os.path
import random
import select
from threading import Semaphore, Thread
import subprocess
import sys
import time

import db

LOGDIR='db/out'
MAX_TIME = 180

ENC_START = '>'*8 + ' start'
ENC_LINE = '-'*8 + ' '
ENC_END = '>'*8 + ' end'

def encode_solution(text):
    src_lines = text.splitlines()
    out = [ENC_START] + [ENC_LINE + i for i in src_lines] + [ENC_END]
    return '\n'.join(out)

def solution_parser():
    solution = None
    while 1:
        s = yield solution
        solution = None
        if s == ENC_START:
            lines = []
            while 1:
                s = yield
                if s.startswith(ENC_LINE):
                    lines.append(s[len(ENC_LINE):])
                if s == ENC_END:
                    solution = '\n'.join(lines)
                    break

def process_solutions(q):
    prefix = 'UPL >>>'
    while 1:
        m = q.get()
        if m is None:
            print prefix, 'exitting...'
            return

        task_id, text = m

        if text is None:
            db.mark_tried_by(task_id, LABEL)
            continue

        db.add_solution(task_id, text)

        if not db.is_solved(task_id):
            while 1:
                print prefix, task_id, 'uploading...'
                reply = db.send_solution_to_orgs(task_id, text)
                r = reply.get('resemblance')
                if r == 1.0:
                    print prefix, 'done, solved!'
                    db.mark_solved_by(task_id, LABEL)
                    break
                else:
                    print prefix, 'solution is rejected!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1'
                    print prefix, reply
                    if 'error' in reply and u'Rate limit exceeded' in reply['error']:
                        print prefix, 'rate limited, will retry later'
                        time.sleep(random.random() * 5)
                    elif 'error' in reply and u'an own problem' in reply['error']:
                        print prefix, 'own task'
                        db.mark_solved_by(task_id, LABEL)
                        break
                    else:
                        break
                time.sleep(2)
        else:
            print prefix, 'task %d is already solved' % task_id


def supervise(task_id, send_cb, done_cb):
    prefix = '%04d --' % task_id

    f = db.taskfile_from_db(task_id)
    if f is None:
        print prefix, 'empty'
        done_cb()
        return

    print prefix, 'start', task_id
    t0 = time.time()
    stdout = file(os.path.join(LOGDIR, '%d-%s.log' % (task_id, LABEL)), 'w')
    stderr = file(os.path.join(LOGDIR, '%d-%s.err' % (task_id, LABEL)), 'w')
    cmd = ['pypy', 'solver.py', str(task_id)]
    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=stderr)

    poller = select.poll()
    poller.register(p.stdout, select.POLLIN)

    out_acc = ''
    parser = solution_parser()
    parser.next()
    killed = False
    while True:
        if poller.poll(1.0):
            data = os.read(p.stdout.fileno(), 1024**2)
            if not data:
                break
            stdout.write(data)

            out_acc += data
            lines = out_acc.split('\n')
            out_acc = lines[-1]
            lines = lines[:-1]
            for s in lines:
                solution_text = parser.send(s)
                if solution_text:
                    print prefix, 'Got solution:'
                    for line in solution_text.splitlines():
                        print prefix, '  ', line
                    send_cb((task_id, solution_text))
        if time.time() - t0 > MAX_TIME:
            print prefix, 'timeout, killing'
            try:
                p.kill()
                killed = True
                break
            except OSError:
                pass
    print prefix, 'end'

    p.wait()
    if p.returncode and not killed:
        print prefix, 'unexpected termination'
    else:
        send_cb((task_id, None))
    done_cb()


def solve_tasks(ids, n_cores):
    queue = Queue.Queue()
    subm = Thread(target=process_solutions, args=(queue,))
    subm.start()

    sema = Semaphore(n_cores)
    try:
        for i in ids:
            if db.is_solved(i):
                continue
            if db.is_tried_by(i, LABEL):
                continue
            sema.acquire()
            t = Thread(target=supervise, args=(i, queue.put, sema.release))
            t.start()
    except KeyboardInterrupt:
        print 'shutting down, please wait...'

    for i in xrange(n_cores): # wait for threads to finish
        sema.acquire()

    print 'workers done...'
    queue.put(None) # stop sender

def main():
    cores = int(sys.argv[1])

    alltasks = db.get_all_task_ids()[::-1]
    new = []
    old = []
    for i in alltasks:
        # FIXME?
        # if os.path.exists(os.path.join(LOGDIR, '%d.log' % i)):
        #     old.append(i)
        # else:
        new.append(i)

    solve_tasks(new+old, cores)
    # supervise(9, lambda: 1)
    
if __name__ == '__main__':
    # alltasks=db.get_all_task_ids()[::-1]
    LABEL = sys.argv[2]
    assert LABEL
    main()
