#!/usr/bin/env python
# coding=utf8
from collections import defaultdict, deque, Counter
import itertools
from pprint import pprint
import sys

from datatypes import Frac, Point, Edge, Polygon, Task,atan3
import datatypes
import harness

def point(xy):
    return Point(*[Frac(s) for s in xy.split()])


def dump_edges(edges, fname=[0], suffix=''):
    if isinstance(fname, list):
        realname = 'debug/%d%s' % (fname[0], suffix)
        fname[0] += 1
    else:
       realname = 'debug/' + fname

    with file(realname, 'w') as f:
        for e in edges:
            print >>f, e.ps[0].format_flt()
            print >>f, e.ps[1].format_flt()
            print >>f
    return realname

def get_permutations(arr):
    for i in len(arr):
        for vs in itertools.combinations(arr, i+1):
            yield vs

# Модные индексы можно добавить позже, когда будет понятно, что надо
# индексировать.
class State(object):
    def __init__(self, polys=[], enumerate_points=True):
        self.area = 0
        self.polys = set()
        # print 'state', polys
        for p in polys:
            self.add_poly(p)

        if enumerate_points:
            self._enumerate_points()

    def clone(self):
        s = State(self.polys, enumerate_points=False)
        s.id_to_dest_point = self.id_to_dest_point # сохраненные точки скелета
        return s

    def add_poly(self, poly):
        self.polys.add(poly)
        self.area += poly.area

    def del_poly(self, poly):
        self.polys.remove(poly)
        self.area -= poly.area

    def _enumerate_points(self):
        all_edges = self.get_all_edge_refs()
        points = {}
        for e in all_edges:
            points[e.ps[0]] = e.ps[0]
            points[e.ps[1]] = e.ps[1]

        for i, p in enumerate(points.values()):
            p.src = i
            # print 'enum', i, p

        for e in all_edges:
            e.ps = (points[e.ps[0]], points[e.ps[1]], e.ps[2])

        self.id_to_dest_point = {p.src: p for p in points.values()}

        #for p in points.values():
        #    print 'ppp', p

        all_edges = self.get_all_edges()
        for e in all_edges:
            assert e.ps[0].src != -1
            assert e.ps[1].src != -1

    def get_all_edge_refs(self):
        edges = set()
        for p in self.polys:
            edges.update(set(p.edges))
        return edges

    def get_all_edges_with_dupes(self):
        edges = []
        for p in self.polys:
            edges.extend(p.edges)
        return edges

    def get_all_edges(self):
        edges = set()
        for p in self.polys:
            for e in p.edges:
                edges.add(e.canonical())
        return edges

    def get_polys_with_edge(self, e):
        polys = set()
        for p in self.polys:
            # print p
            for e_ in p.edges:
                if e.eq_undirectional(e_):
                    polys.add(p)
        return polys

    def get_neighbour_polys(self, polygon):
        polys = set()
        for e in polygon.edges:
            polys.update(self.get_polys_with_edge(e))
        return polys - set([polygon])

    # Генерируем последовательность полигонов начиная с края edge методом волны.
    # На каждой итерации получаем лист полиномов, которые хотим перевернуть.
    def get_edge_wave_iter(self, edge):
        walked_polys = set()
        front_polys = deque(self.get_polys_with_edge(edge))
        while front_polys:
            # тут присутствуют не все пермутации, видимо поэтому №14 не решается
            p = front_polys.popleft()
            walked_polys.add(p)
            yield walked_polys
            front_polys.extend(self.get_neighbour_polys(p) - walked_polys)

    def get_mirror_poly_iter(self, edge):
        tmp = self.get_polys_with_edge(edge)
        assert len(tmp) == 1
        root = list(tmp)[0]

        def path_to_hashable(path):
            ids = [id(p) for p in path]
            ids.sort()
            return tuple(ids)

        seen = set()
        def breadth_walk(path):
            edge = set() # contains Polygons (not Edges)
            for p in path:
                edge.update(set(self.get_neighbour_polys(p)))
            edge = edge - path

            spawn_later = []
            for p in edge:
                path.add(p)
                path_hash = path_to_hashable(path)
                if path_hash not in seen:
                    seen.add(path_hash)
                    spawn_later.append(p)
                    yield path
                path.remove(p)

            for p in spawn_later:
                path.add(p)
                for x in breadth_walk(path):
                    yield x
                path.remove(p)

        yield set([root])
        for x in breadth_walk(set([root])):
            yield x

    def get_border_if_planar(self):
        # TODO: perf: can return early
        all_edges = self.get_all_edges_with_dupes()
        canonical = [e.canonical() for e in all_edges]
        vecs = [(e.ps[0], e.ps[1]) for e in canonical]
        c = Counter(vecs)

        border = []
        planar = True
        #print 'count', c
        for (a, b), n in c.items():
            # print n
            if n == 1:
                border.append(Edge(a, b))
            elif n != 2:
                return None
        # for p in self.polys:
        #    print 'poly', p
        # print 'nvecs', len(vecs)
        # print 'nbord', len(border)
        # for v in vecs:
            # print 'vecs:', v
        # for b in border:
            # print 'border:', b
        # print 'dump', dump_edges(border, suffix='border')
        assert len(set(border)) == len(border)
        return border
        

    def get_outline(self):

        alledges=self.get_border_if_planar()
        if not alledges:
            return None

        startedge = alledges[0]

        points=set()
        for i in alledges:
            points.add(i.ps[0])
            points.add(i.ps[1])

        pedges={}
        for i in points:
            pedges[i]=[]
            for j in alledges:
                if i in j.ps[:2]:
                    pedges[i].append(j)
                if len(pedges[i]) > 2:
                    # print i, pedges[i]
                    return None

        for i in pedges.keys():
            if len(pedges[i]) != 2:
                return None

        curredge=startedge
        currpoint=curredge.ps[1]
        
        #сначала налево
        trace=[curredge]
        
        while True:
            curredges=pedges[currpoint]

            if curredges[0].eq_geo(curredge):
                nextedge=curredges[1]
            else:
                nextedge=curredges[0]
            
            if nextedge.ps[0]==currpoint:
                currpoint=nextedge.ps[1]
            else:
                currpoint=nextedge.ps[0]
            if nextedge.eq_geo(startedge):
                break
            trace.append(nextedge)
            curredge=nextedge

        # for b in trace:
        #    print 'trace:', b
        #assert len(set(trace)) == len(trace)
        return trace
         
    def is_square(self):
        outline=self.get_outline()
        if not outline or len(outline) < 4:
            # print "no outline", outline
            return False
        # else:
            # print "orig",outline,len(outline)
        joined=True    
        while joined:
            joined=False
            newoutline=[]
            for i in xrange(-1,len(outline)-1):
                if outline[i].is_parallel(outline[i+1]):
                    #print '---',i,'---',len(outline)
                    #print outline
                    if outline[i].eq_undirectional(outline[i+1]):
                        print 'fold back'
                        # print 'dump', dump_edges(outline, suffix='outline')
                        return None
                    if outline[i].ps[0] == outline[i+1].ps[0]:
                        newedge = Edge(outline[i].ps[1], outline[i+1].ps[1])
                    elif outline[i].ps[0] == outline[i+1].ps[1]:
                        newedge = Edge(outline[i].ps[1], outline[i+1].ps[0])
                    elif outline[i].ps[1] == outline[i+1].ps[0]:
                        newedge = Edge(outline[i].ps[0], outline[i+1].ps[1])
                    elif outline[i].ps[1] == outline[i+1].ps[1]:
                        newedge = Edge(outline[i].ps[0], outline[i+1].ps[0])
                    else:
                        # print "collapse",outline[i].ps,outline[i+1].ps
                        return None
                    # print "==joining",i, len(outline), outline[i],outline[i+1],newedge
                    joined=True    
                    
                        
                    newoutline.append(newedge)
                    # print '  appended', newoutline
                    newoutline.extend(outline[i+2:-1])
                    # print '  extend with', outline[i+2:-1]
                    if i==len(outline)-2:
                        newoutline=newoutline[1:]
                    # assert len(newoutline) + 1 == len(outline)
                    
                    break
                else:
                    newoutline.append(outline[i])
                
                
            outline=newoutline
        
        if len(outline)!=4:
            # print "not 4 vertices", len(outline)
            return False
        for i in range(-1,3):
            if outline[i].scalar(outline[i+1])!=0:
                print "not rect"
                return False
            if outline[i].scalar(outline[i])!=1:
                # print "not 1", outline[i].scalar(outline[i])
                # print 'dump', outline
                # print 'dump', dump_edges(outline, suffix='outlinenot1')
                return False
        #Это понадобится для трансформации
        
        self.outline=outline
        
        return True
    def canon_coords(self,p):
        
        
        if self.outline[0].ps[0] in self.outline[1].ps[:2]:
            nk=self.outline[0].ps[0]
            v1=(self.outline[0].ps[1][0]-self.outline[0].ps[0][0],self.outline[0].ps[1][1]-self.outline[0].ps[0][1])
            
        else:
            nk=self.outline[0].ps[1]
            v1=(self.outline[0].ps[0][0]-self.outline[0].ps[1][0],self.outline[0].ps[0][1]-self.outline[0].ps[1][1])
        if nk==self.outline[1].ps[0]:
            v2=(self.outline[1].ps[1][0]-self.outline[1].ps[0][0],self.outline[1].ps[1][1]-self.outline[1].ps[0][1])
        else:
            v2=(self.outline[1].ps[0][0]-self.outline[1].ps[1][0],self.outline[1].ps[0][1]-self.outline[1].ps[1][1])
        loc=(p[0]-nk[0],p[1]-nk[1])
        return Point(loc[0]*v1[0]+loc[1]*v1[1],loc[0]*v2[0]+loc[1]*v2[1])
        
    def has_tears_in_solution(self):
        point_to_src = {}
        for e in self.get_all_edges_with_dupes():
            for p in e.ps[:2]:
                src = point_to_src.get(p)
                if src is None:
                    # print 'ins'
                    point_to_src[p] = p.src
                elif src != p.src:
                    # print '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! tear'
                    # assert False
                    return True
                # else:
                    # print 'match'
                
        return False

    def is_solution(self):
        if self.area != 1:
            return False

        if self.has_tears_in_solution():
            return False
        
        if not self.is_square():
            return False
        # TODO: insert proper checks
        # проверить на несовпадающие точки в развернутом варианте
        return True

        
    def dump_solution(self):
        all_edges = self.get_all_edges()

        source_point_to_id = {}
        for e in all_edges:
            source_point_to_id[e.ps[0]] = 0
            source_point_to_id[e.ps[1]] = 0
        for i, k in enumerate(source_point_to_id.keys()):
            source_point_to_id[k] = i

        lines = []
        # точки на развертке
        id_and_point = [(i, p) for p, i in source_point_to_id.items()]
        id_and_point.sort()
        lines.append(str(len(source_point_to_id)))
        for i, p in id_and_point:
            # print 'src:', i, p
            lines.append(self.canon_coords(p).format())

        # полигоны
        lines.append(str(len(self.polys)))
        for poly in self.polys:
            # print poly
            edge_nums = [len(poly.edges)]
            for e in poly.edges:
                edge_nums.append(source_point_to_id[e.ps[0]])
            lines.append(' '.join(str(i) for i in edge_nums))

        # точки на свертке
        # print self.id_to_dest_point
        for p in source_point_to_id.keys():
            p_src = self.id_to_dest_point[p.src]
            lines.append(p_src.format())

        return '\n'.join(lines)

def mirror_poly(edge, poly, zindex):
    edges = []
    for e in poly.edges:
        if not e.eq_undirectional(edge):
            pt0 = edge.mirror_point(e.ps[0])
            pt1 = edge.mirror_point(e.ps[1])
            e = Edge(pt0, pt1, zindex)
        edges.append(e)
    return Polygon(edges, area=poly.area)

def solve(state, depth=0, debug=False):
    if debug:
        state_polys = [state]
    elif state.is_solution():
        print '!!!!!! solved'
        print harness.encode_solution(state.dump_solution())
        sys.stdout.flush()
        state_polys = [state]
    else:
        state_polys = []
    # print '      area', state.area

    # print '====STATE POLYS'
    # for p in state.polys:
    #     print '    ', p
    # print
    if depth == 3:
        return state_polys

    for e in state.get_all_edges():
        # print e
        # if not e.eq_undirectional(Edge(point("1 1/2"), point("0 3/4"))):
        #     continue

        edge_polys = state.get_polys_with_edge(e)
        if len(edge_polys) != 1:
            continue

        root_poly = list(edge_polys)[0]
        # print ' pnum =', len(edge_polys)

        for polys in itertools.islice(state.get_mirror_poly_iter(e), 5):
        # for polys in state.get_mirror_poly_iter(e):
            # print '   ', len(polys), polys

            # разворачиваем и рекурсивно спускаемся
            unfolded_state = state.clone()
            for p in polys:
                mirrored = mirror_poly(e, p, depth + 1)
                unfolded_state.add_poly(mirrored)

            # print unfolded_state.area
            # if unfolded_state.area == 1:
                # print 'area = 1'

            if unfolded_state.area <= 1:
                state_polys.extend(solve(unfolded_state, depth + 1, debug=debug))

            # попробуем поудалять из исходного оригами развернутые куски
            # пока удаляем только один кусок
            for p in polys:
                # print '  rm', unfolded_state.area - p.area, p
                if p == root_poly: # чтобы не получить сразу дыру
                    # print '    skipping root'
                    continue
                if unfolded_state.area - p.area > 1:
                    # print '    too large'
                    continue
                # print '    processing'
                hole_state = unfolded_state.clone()
                hole_state.del_poly(p)
                state_polys.extend(solve(hole_state, depth + 1, debug=debug))

    return state_polys

def test():
    p1 = point('0.0 0.0')
    p2 = point('0.0 1.0')
    p3 = point('0.7 1.0')
    p4 = point('1.0 0.7')
    p5 = point('1.0 0.0')
    p6 = point('0.7 0.7')

    poly1 = Polygon.from_points([p1, p2, p3, p6, p4, p5])
    poly2 = Polygon.from_points([p3, p6, p4])
    # solve([poly1, poly2])

    s = State([poly1, poly2])
    print s.get_polys_with_edge(Edge(p1, p2))
    assert s.get_polys_with_edge(Edge(p1, p2)) == set([poly1])
    assert s.get_polys_with_edge(Edge(p3, p4)) == set([poly2])
    assert s.get_polys_with_edge(Edge(p3, p6)) == set([poly1, poly2])
    assert s.get_neighbour_polys(poly1) == set([poly2])
    assert s.get_neighbour_polys(poly2) == set([poly1])

    for state in solve(State([poly1, poly2])):
        print s.polys

def run_task(n):
    import db
    task = db.task_from_db(n)
    tstate = datatypes.State(task)
    variants = solve(State(tstate.polys))
    for i, state in enumerate(variants):
        print '-----', i
        for p in state.polys:
            print '  ', p

        print '>>>>>>>'
        print state.dump_solution()


if __name__ == '__main__':
    import sys
    if len(sys.argv) == 1:
        test()
    else:
        run_task(sys.argv[1])
